@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -26,6 +26,7 @@
 #include <gtk/gtk.h>
 #include <gmime/gmime.h>
 #include "libbalsa-gpgme.h"
+#include "mime-stream-shared.h"
 #include "gmime-part-rfc2440.h"
 
 
@@ -70,6 +71,9 @@ g_mime_part_check_rfc2440(GMimePart * part)
     if (!stream || (slen = g_mime_stream_length(stream)) < 0)
 	return retval;
 
+    /* note: the following is a noop if the stream is not a shared stream */
+    libbalsa_mime_stream_shared_lock(stream);
+
     g_mime_stream_reset(stream);
 
     /* check if the complete stream fits in the buffer */
@@ -110,6 +114,8 @@ g_mime_part_check_rfc2440(GMimePart * part)
 	}
     }
 
+    libbalsa_mime_stream_shared_unlock(stream);
+
     return retval;
 }
 
@@ -187,7 +193,7 @@ g_mime_part_rfc2440_sign_encrypt(GMimePart * part, const char *sign_userid,
 	rfc2440header =
 	    g_strdup_printf("-----BEGIN PGP MESSAGE-----\nCharset: %s\n"
 			    "Comment: created by Balsa " BALSA_VERSION
-			    " (http://balsa.gnome.org)", charset);
+			    " (https://pawsa.fedorapeople.org/balsa)", charset);
 	g_byte_array_remove_range(cipherdata, 0, 27);
 	g_byte_array_prepend(cipherdata, (guint8 *) rfc2440header,
 			     strlen(rfc2440header));
