@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -243,8 +243,6 @@ libbalsa_mailbox_maildir_new(const gchar * path, gboolean create)
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MAILDIR, NULL);
 
-    libbalsa_mailbox_set_is_directory(mailbox, TRUE);
-
     if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
                                         path, create) != 0) {
         g_object_unref(mailbox);
@@ -363,8 +361,10 @@ static LibBalsaMessageFlag parse_filename(const gchar *subdir,
     return flags;
 }
 
-static void lbm_maildir_parse(LibBalsaMailboxMaildir * mdir,
-                              const gchar *subdir, guint * fileno)
+static void
+lbm_maildir_parse(LibBalsaMailboxMaildir *mdir,
+                  const gchar            *subdir,
+                  guint                  *fileno)
 {
     const gchar *local_path;
     gchar *path;
@@ -578,9 +578,10 @@ libbalsa_mailbox_maildir_check(LibBalsaMailbox * mailbox)
             libbalsa_message_set_msgno(msg_info->local_info.message, msgno);
     }
 
-    msgno = mdir->msgno_2_msg_info->len;
     lbm_maildir_parse_subdirs(mdir);
-    libbalsa_mailbox_local_load_messages(mailbox, msgno);
+
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->check != NULL)
+        LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_maildir_parent_class)->check(mailbox);
 }
 
 static void
