@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -132,7 +132,7 @@ libbalsa_mailbox_mbox_class_init(LibBalsaMailboxMboxClass * klass)
     libbalsa_mailbox_class->lock_store = libbalsa_mailbox_mbox_lock_store;
 
     libbalsa_mailbox_local_class->check_files  = lbm_mbox_check_files;
-    libbalsa_mailbox_local_class->remove_files = 
+    libbalsa_mailbox_local_class->remove_files =
 	libbalsa_mailbox_mbox_remove_files;
 
     libbalsa_mailbox_local_class->get_info = lbm_mbox_get_info;
@@ -192,8 +192,6 @@ libbalsa_mailbox_mbox_new(const gchar * path, gboolean create)
 
     mailbox = g_object_new(LIBBALSA_TYPE_MAILBOX_MBOX, NULL);
 
-    libbalsa_mailbox_set_is_directory(mailbox, FALSE);
-
     if (libbalsa_mailbox_local_set_path(LIBBALSA_MAILBOX_LOCAL(mailbox),
                                         path, create) != 0) {
 	g_object_unref(mailbox);
@@ -430,7 +428,6 @@ static LibBalsaMessage *lbm_mbox_message_new(GMimeMessage * mime_message,
 static void
 parse_mailbox(LibBalsaMailboxMbox * mbox)
 {
-    LibBalsaMailboxLocal *local = LIBBALSA_MAILBOX_LOCAL(mbox);
     GMimeParser *gmime_parser;
     struct message_info msg_info;
     struct message_info * msg_info_p = &msg_info;
@@ -443,7 +440,6 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
                                    "^Status|^X-Status|^MIME-Version",
 				   lbm_mbox_header_cb, &msg_info_p);
 
-    libbalsa_mailbox_local_set_threading_info(local);
     msg_info.local_info.message = NULL;
     msg_info.local_info.loaded  = FALSE;
     while (!g_mime_parser_eos(gmime_parser)) {
@@ -516,7 +512,7 @@ parse_mailbox(LibBalsaMailboxMbox * mbox)
 	 * holding the lock. */
         offset = g_mime_stream_tell(mbox->gmime_stream);
         libbalsa_mime_stream_shared_unlock(mbox->gmime_stream);
-        libbalsa_mailbox_local_cache_message(local, msgno, msg);
+        libbalsa_mailbox_cache_message(LIBBALSA_MAILBOX(mbox), msgno, msg);
         libbalsa_mime_stream_shared_lock(mbox->gmime_stream);
         g_mime_stream_seek(mbox->gmime_stream, offset, GMIME_STREAM_SEEK_SET);
 
@@ -1076,7 +1072,9 @@ libbalsa_mailbox_mbox_check(LibBalsaMailbox * mailbox)
 #endif
     libbalsa_mime_stream_shared_unlock(mbox_stream);
     mbox_unlock(mailbox, mbox_stream);
-    libbalsa_mailbox_local_load_messages(mailbox, msgno);
+
+    if (LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mbox_parent_class)->check != NULL)
+        LIBBALSA_MAILBOX_CLASS(libbalsa_mailbox_mbox_parent_class)->check(mailbox);
 }
 
 static void
