@@ -14,7 +14,7 @@
  * GNU General Public License for more details.
  *  
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, see <http://www.gnu.org/licenses/>.
+ * along with this program; if not, see <https://www.gnu.org/licenses/>.
  */
 
 #if defined(HAVE_CONFIG_H) && HAVE_CONFIG_H
@@ -131,29 +131,29 @@ static LibBalsaMessageBody *
 find_alt_part(LibBalsaMessageBody *parts,
 			  gboolean			   print_alt_html)
 {
-	LibBalsaMessageBody *use_part;
+	LibBalsaMessageBody *part;
+	LibBalsaMessageBody *use_part = parts;		/* fallback: print 1st alternative part */
 
 	/* scan the parts */
-	for (use_part = parts; use_part != NULL; use_part = use_part->next) {
+	for (part = parts; part != NULL; part = part->next) {
 		gchar *mime_type;
 		
-		mime_type = libbalsa_message_body_get_mime_type(use_part);
-		if ((g_ascii_strncasecmp(mime_type, "multipart/", 10U) == 0) && (use_part->parts != NULL)) {
+		mime_type = libbalsa_message_body_get_mime_type(part);
+		if ((strncmp(mime_type, "multipart/", 10U) == 0) && (part->parts != NULL)) {
 			/* consider the first child of a multipart */
 			g_free(mime_type);
-			mime_type = libbalsa_message_body_get_mime_type(use_part->parts);
+			mime_type = libbalsa_message_body_get_mime_type(part->parts);
 		}
 
-		if (((g_ascii_strcasecmp(mime_type, "text/plain") == 0) && !print_alt_html) ||
-			((g_ascii_strcasecmp(mime_type, "text/html") == 0) && print_alt_html)) {
-			g_free(mime_type);
-			return use_part;
+		if ((strcmp(mime_type, "text/calendar") == 0) ||
+			((strcmp(mime_type, "text/plain") == 0) && !print_alt_html) ||
+			((strcmp(mime_type, "text/html") == 0) && print_alt_html)) {
+			use_part = part;
 		}
 		g_free(mime_type);
 	}
 	
-	/* nothing found, fall back to the first part in the chain */
-	return parts;
+	return use_part;
 }
 
 
@@ -211,9 +211,9 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 
 	conttype = libbalsa_message_body_get_mime_type(body);
 	add_signature = body->sig_info &&
-	    g_ascii_strcasecmp(conttype, "application/pgp-signature") &&
-	    g_ascii_strcasecmp(conttype, "application/pkcs7-signature") &&
-	    g_ascii_strcasecmp(conttype, "application/x-pkcs7-signature");
+	    strcmp(conttype, "application/pgp-signature") != 0 &&
+	    strcmp(conttype, "application/pkcs7-signature") != 0 &&
+	    strcmp(conttype, "application/x-pkcs7-signature") != 0;
 
 	is_mp_signed = libbalsa_message_body_multipart_signed(body);
 	if (is_mp_signed || (!add_signature && body->was_encrypted)) {
@@ -242,6 +242,11 @@ scan_body(GList *bpo_list, GtkPrintContext * context, BalsaPrintSetup * psetup,
 
 			print_part = find_alt_part(body->parts, psetup->print_alt_html);
 			bpo_list = print_single_part(bpo_list, context, psetup, print_part, no_first_sep, add_signature);
+		} else if (g_ascii_strcasecmp(conttype, "multipart/related") == 0) {
+			/* catch the case of a RFC 2387 multipart/related, typically a text/html with images which are enclosed in the
+			 * "related" container */
+			bpo_list = print_single_part(bpo_list, context, psetup, libbalsa_message_body_mp_related_root(body), no_first_sep,
+				add_signature);
 		} else {
 			bpo_list = scan_body(bpo_list, context, psetup, body->parts, no_first_sep);
 		}
@@ -471,9 +476,9 @@ add_font_button(const gchar * text, const gchar * font, GtkGrid * grid,
     GtkWidget *font_button;
 
     label = gtk_label_new_with_mnemonic(text);
-    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0F);
     gtk_grid_attach(grid, label, 1, row, 1, 1);
 
     font_button = gtk_font_button_new_with_font(font);
@@ -494,9 +499,9 @@ add_margin_spinbtn(const gchar * text, gdouble min, gdouble max, gdouble dflt,
     const gchar *unit;
 
     label = gtk_label_new_with_mnemonic(text);
-    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
     gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
+    gtk_label_set_xalign(GTK_LABEL(label), 0.0F);
     gtk_grid_attach(grid, label, 1, row, 1, 1);
 
     if (get_default_user_units() == GTK_UNIT_INCH) {
@@ -519,8 +524,6 @@ add_margin_spinbtn(const gchar * text, gdouble min, gdouble max, gdouble dflt,
     gtk_grid_attach(grid, spinbtn, 2, row, 1, 1);
 
     label = gtk_label_new(unit);
-    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
     gtk_widget_set_halign(label, GTK_ALIGN_START);
     gtk_grid_attach(grid, label, 3, row, 1, 1);
 
@@ -549,12 +552,9 @@ create_options_group(const gchar *label_str, GtkWidget *parent_grid, gint parent
     group = gtk_box_new(GTK_ORIENTATION_VERTICAL, 12);
     gtk_grid_attach(GTK_GRID(parent_grid), group, parent_col, parent_row, parent_width, 1);
 
-    label = gtk_label_new(NULL);
     markup = g_strdup_printf("<b>%s</b>", label_str);
-    gtk_label_set_markup(GTK_LABEL(label), markup);
+    label = libbalsa_create_wrap_label(markup, TRUE);
     g_free(markup);
-    gtk_label_set_line_wrap(GTK_LABEL(label), TRUE);
-    gtk_widget_set_halign(label, GTK_ALIGN_START);
     gtk_box_pack_start(GTK_BOX(group), label, FALSE, FALSE, 0);
 
 	grid = gtk_grid_new();
